"""Forms for FIDO 2 registration and login."""
import base64

from django import forms
from django.contrib.auth import authenticate
from django.contrib.auth.forms import AuthenticationForm
from django.core.exceptions import ValidationError
from django.http import HttpRequest
from django.utils.translation import gettext_lazy as _
from fido2.client import ClientData
from fido2.ctap2 import AttestationObject, AuthenticatorData
from fido2.server import Fido2Server


class Fido2RegistrationForm(forms.Form):
    """Form for FIDO 2 registration responses."""

    client_data = forms.CharField(error_messages={'required': _("Operation wasn't completed.")},
                                  widget=forms.HiddenInput)
    attestation = forms.CharField(error_messages={'required': _("Operation wasn't completed.")},
                                  widget=forms.HiddenInput)
    label = forms.CharField(required=False, max_length=255, label=_("Label"))

    class Media:
        """Add FIDO 2 related JS."""

        js = ('django_fido/js/fido2.js', )

    def clean_client_data(self) -> ClientData:
        """Return decoded client data."""
        value = self.cleaned_data['client_data']
        try:
            return ClientData(base64.b64decode(value))
        except ValueError:
            raise ValidationError(_('FIDO 2 response is malformed.'), code='invalid')

    def clean_attestation(self) -> AttestationObject:
        """Return decoded attestation object."""
        value = self.cleaned_data['attestation']
        try:
            return AttestationObject(base64.b64decode(value))
        except ValueError:
            raise ValidationError(_('FIDO 2 response is malformed.'), code='invalid')


class Fido2AuthenticationForm(forms.Form):
    """Form for FIDO 2 authentication responses."""

    client_data = forms.CharField(error_messages={'required': _("Operation wasn't completed.")},
                                  widget=forms.HiddenInput)
    credential_id = forms.CharField(error_messages={'required': _("Operation wasn't completed.")},
                                    widget=forms.HiddenInput)
    authenticator_data = forms.CharField(error_messages={'required': _("Operation wasn't completed.")},
                                         widget=forms.HiddenInput)
    signature = forms.CharField(error_messages={'required': _("Operation wasn't completed.")},
                                widget=forms.HiddenInput)

    class Media:
        """Add FIDO 2 related JS."""

        js = ('django_fido/js/fido2.js', )

    def clean_client_data(self) -> ClientData:
        """Return decoded client data."""
        value = self.cleaned_data['client_data']
        try:
            return ClientData(base64.b64decode(value))
        except ValueError:
            raise ValidationError(_('FIDO 2 response is malformed.'), code='invalid')

    def clean_credential_id(self) -> bytes:
        """Return decoded credential ID."""
        value = self.cleaned_data['credential_id']
        try:
            return base64.b64decode(value)
        except ValueError:
            raise ValidationError(_('FIDO 2 response is malformed.'), code='invalid')

    def clean_authenticator_data(self) -> AuthenticatorData:
        """Return decoded authenticator data."""
        value = self.cleaned_data['authenticator_data']
        try:
            return AuthenticatorData(base64.b64decode(value))
        except ValueError:
            raise ValidationError(_('FIDO 2 response is malformed.'), code='invalid')

    def clean_signature(self) -> bytes:
        """Return decoded signature."""
        value = self.cleaned_data['signature']
        try:
            return base64.b64decode(value)
        except ValueError:
            raise ValidationError(_('FIDO 2 response is malformed.'), code='invalid')


class Fido2ModelAuthenticationForm(AuthenticationForm, Fido2AuthenticationForm):
    """Authentication form with username, password and FIDO 2 credentials."""

    def __init__(self, request: HttpRequest, fido2_server: Fido2Server, session_key: str, *args, **kwargs):
        """Initialize form."""
        super().__init__(request, *args, **kwargs)
        self.fido2_server = fido2_server
        self.session_key = session_key
        self.error_messages['invalid_login'] = _(
            "Please enter a correct %(username)s and password and use valid FIDO2 security key."
        )

    def clean(self):
        """Authenticate user."""
        username = self.cleaned_data.get('username')
        password = self.cleaned_data.get('password')
        state = self.request.session.pop(self.session_key, None)
        if state is None:
            raise ValidationError(_('Authentication request not found.'), code='missing')

        self.user_cache = authenticate(
            self.request,
            username=username,
            password=password,
            fido2_server=self.fido2_server,
            fido2_state=state,
            fido2_response=self.cleaned_data,
        )
        if self.user_cache is None:
            raise self.get_invalid_login_error()
        else:
            self.confirm_login_allowed(self.user_cache)

        return self.cleaned_data

    def get_invalid_login_error(self):
        """Get invalid login error."""
        # FIXME: This method of AuthenticationForm was first introduced in Django 2.1.
        # As soon as we stop supporting lower Django versions, we can delete it.
        return forms.ValidationError(
            self.error_messages['invalid_login'],
            code='invalid_login',
            params={'username': self.username_field.verbose_name},
        )



"""Test `django_fido.views` module."""
import base64
import json

import fido2
from django.contrib.auth import get_user, get_user_model
from django.core.exceptions import NON_FIELD_ERRORS
from django.test import TestCase, override_settings
from django.urls import reverse, reverse_lazy
from fido2.utils import websafe_decode

from django_fido.constants import AUTHENTICATION_USER_SESSION_KEY, FIDO2_REQUEST_SESSION_KEY
from django_fido.models import Authenticator

from .data import (ATTESTATION_OBJECT, ATTESTATION_OBJECT_BOGUS, ATTESTATION_OBJECT_U2F_MALFORMED,
                   AUTHENTICATION_CHALLENGE, AUTHENTICATION_CLIENT_DATA, AUTHENTICATOR_DATA, CREDENTIAL_ID, HOSTNAME,
                   PASSWORD, REGISTRATION_CHALLENGE, REGISTRATION_CLIENT_DATA, SIGNATURE, USER_FIRST_NAME,
                   USER_FULL_NAME, USER_LAST_NAME, USERNAME)
from .utils import TEMPLATES

try:
    from fido2.webauthn import UserVerificationRequirement
except ImportError:
    from fido2.server import USER_VERIFICATION as UserVerificationRequirement

User = get_user_model()


@override_settings(ROOT_URLCONF='django_fido.tests.urls')
class TestFido2RegistrationRequestView(TestCase):
    """Test `Fido2RegistrationRequestView` class."""

    url = reverse_lazy('django_fido:registration_request')

    def setUp(self):
        self.user = User.objects.create_user(USERNAME, first_name=USER_FIRST_NAME, last_name=USER_LAST_NAME)

    def test_anonymous(self):
        with self.settings(LOGIN_URL='/login/'):
            response = self.client.get(self.url)

        self.assertRedirects(response, '/login/?next={}'.format(self.url), fetch_redirect_response=False)

    def _get_fido2_request(self, challenge, credentials):
        credential_params = [{'alg': -7, 'type': 'public-key'}, {'alg': -8, 'type': 'public-key'},
                             {'alg': -37, 'type': 'public-key'}, {'alg': -257, 'type': 'public-key'}]
        rp_data = {
            'id': HOSTNAME,
            'name': HOSTNAME,
        }
        fido2_request = {'publicKey': {
            'rp': rp_data,
            'user': {'displayName': USER_FULL_NAME, 'id': USERNAME, 'name': USERNAME},
            'timeout': 30000,
            'challenge': base64.b64encode(challenge).decode('utf-8'),
            'pubKeyCredParams': credential_params,
            'attestation': 'none',
            'excludeCredentials': credentials,
        }}
        if fido2.__version__ < '0.8':
            fido2_request['publicKey']['authenticatorSelection'] = {'requireResidentKey': False,
                                                                    'userVerification': 'preferred'}
        return fido2_request

    def test_get(self):
        self.client.force_login(self.user)

        response = self.client.get(self.url)

        self.assertEqual(response.status_code, 200)
        # Check response
        state = self.client.session[FIDO2_REQUEST_SESSION_KEY]
        challenge = websafe_decode(state['challenge'])
        self.assertEqual(response.json(), self._get_fido2_request(challenge, []))

    def test_get_registered_keys(self):
        Authenticator.objects.create(user=self.user, attestation_data=ATTESTATION_OBJECT)
        self.client.force_login(self.user)

        response = self.client.get(self.url)

        self.assertEqual(response.status_code, 200)
        # Check response contains the same request as session
        state = self.client.session[FIDO2_REQUEST_SESSION_KEY]
        challenge = websafe_decode(state['challenge'])
        credentials = [{'id': CREDENTIAL_ID, 'type': 'public-key'}]
        self.assertEqual(response.json(), self._get_fido2_request(challenge, credentials))


@override_settings(ROOT_URLCONF='django_fido.tests.urls', TEMPLATES=TEMPLATES)
class TestFido2RegistrationView(TestCase):
    """Test `Fido2RegistrationView` class."""

    url = reverse_lazy('django_fido:registration')
    state = {'challenge': REGISTRATION_CHALLENGE, 'user_verification': UserVerificationRequirement.PREFERRED}

    def setUp(self):
        self.user = User.objects.create_user(USERNAME)

    def test_anonymous(self):
        with self.settings(LOGIN_URL='/login/'):
            response = self.client.get(self.url)

        self.assertRedirects(response, '/login/?next={}'.format(self.url), fetch_redirect_response=False)

    def test_get(self):
        self.client.force_login(self.user)

        response = self.client.get(self.url)

        self.assertContains(response, 'Register a new FIDO 2 authenticator')

    def test_post(self):
        self.client.force_login(self.user)
        session = self.client.session
        session[FIDO2_REQUEST_SESSION_KEY] = self.state
        session.save()

        response = self.client.post(self.url,
                                    {'client_data': REGISTRATION_CLIENT_DATA, 'attestation': ATTESTATION_OBJECT})

        self.assertRedirects(response, reverse('django_fido:registration_done'))
        queryset = Authenticator.objects.values_list('user__pk', 'credential_id_data', 'attestation_data', 'counter')
        key_data = (self.user.pk, CREDENTIAL_ID, ATTESTATION_OBJECT, 0)
        self.assertQuerysetEqual(queryset, [key_data], transform=tuple)
        self.assertNotIn(FIDO2_REQUEST_SESSION_KEY, self.client.session)

    def test_post_no_session(self):
        self.client.force_login(self.user)

        response = self.client.post(self.url,
                                    {'client_data': REGISTRATION_CLIENT_DATA, 'attestation': ATTESTATION_OBJECT})

        self.assertContains(response, 'Register a new FIDO 2 authenticator')
        self.assertEqual(response.context['form'].errors, {NON_FIELD_ERRORS: ['Registration request not found.']})
        self.assertNotIn(FIDO2_REQUEST_SESSION_KEY, self.client.session)

    def test_post_invalid_response(self):
        self.client.force_login(self.user)
        session = self.client.session
        session[FIDO2_REQUEST_SESSION_KEY] = self.state
        session.save()

        client_data = base64.b64encode(json.dumps({"type": "invalid"}).encode()).decode()
        response = self.client.post(self.url, {'client_data': client_data, 'attestation': ATTESTATION_OBJECT})

        self.assertContains(response, 'Register a new FIDO 2 authenticator')
        self.assertEqual(response.context['form'].errors, {NON_FIELD_ERRORS: ['Registration failed.']})
        self.assertNotIn(FIDO2_REQUEST_SESSION_KEY, self.client.session)

    def test_post_invalid_format(self):
        self.client.force_login(self.user)
        session = self.client.session
        session[FIDO2_REQUEST_SESSION_KEY] = self.state
        session.save()

        url = reverse_lazy('registration_direct')
        response = self.client.post(url, {'client_data': REGISTRATION_CLIENT_DATA,
                                          'attestation': ATTESTATION_OBJECT_BOGUS})

        self.assertContains(response, 'Register a new FIDO 2 authenticator')
        self.assertEqual(response.context['form'].errors,
                         {NON_FIELD_ERRORS: ['Security key is not supported because it cannot be identified.']})
        self.assertNotIn(FIDO2_REQUEST_SESSION_KEY, self.client.session)

    def test_post_invalid_data(self):
        self.client.force_login(self.user)
        session = self.client.session
        session[FIDO2_REQUEST_SESSION_KEY] = self.state
        session.save()

        url = reverse_lazy('registration_direct')
        response = self.client.post(url, {'client_data': REGISTRATION_CLIENT_DATA,
                                          'attestation': ATTESTATION_OBJECT_U2F_MALFORMED})

        self.assertContains(response, 'Register a new FIDO 2 authenticator')
        self.assertEqual(response.context['form'].errors,
                         {NON_FIELD_ERRORS: ['Registration failed, incorrect data from security key.']})
        self.assertNotIn(FIDO2_REQUEST_SESSION_KEY, self.client.session)


@override_settings(ROOT_URLCONF='django_fido.tests.urls')
class TestFido2AuthenticationRequestView(TestCase):
    """Test `Fido2AuthenticationRequestView` class."""

    url = reverse_lazy('django_fido:authentication_request')

    def setUp(self):
        self.user = User.objects.create_user(USERNAME)

    def test_no_user(self):
        with self.settings(LOGIN_URL='/login/'):
            response = self.client.get(self.url)

        self.assertRedirects(response, '/login/', fetch_redirect_response=False)

    @override_settings(DJANGO_FIDO_TWO_STEP_AUTH=True)
    def test_get(self):
        Authenticator.objects.create(user=self.user, attestation_data=ATTESTATION_OBJECT)
        session = self.client.session
        session[AUTHENTICATION_USER_SESSION_KEY] = self.user.pk
        session.save()

        response = self.client.get(self.url)

        self.assertEqual(response.status_code, 200)
        # Check response
        state = self.client.session[FIDO2_REQUEST_SESSION_KEY]
        challenge = websafe_decode(state['challenge'])
        fido2_request = {
            'publicKey': {'rpId': 'testserver',
                          'challenge': base64.b64encode(challenge).decode('utf-8'),
                          'allowCredentials': [{'id': CREDENTIAL_ID, 'type': 'public-key'}],
                          'timeout': 30000}}
        if fido2.__version__ < '0.8':
            fido2_request['publicKey']['userVerification'] = 'preferred'
        self.assertEqual(response.json(), fido2_request)

    @override_settings(DJANGO_FIDO_TWO_STEP_AUTH=True)
    def test_get_no_keys(self):
        session = self.client.session
        session[AUTHENTICATION_USER_SESSION_KEY] = self.user.pk
        session.save()

        response = self.client.get(self.url)

        self.assertEqual(response.status_code, 400)
        self.assertEqual(response.json(), {
            'error_code': 'NoAuthenticatorsError',
            'message': "Can't create FIDO 2 authentication request, no authenticators found.",
            'error': "Can't create FIDO 2 authentication request, no authenticators found.",
        })
        self.assertNotIn(FIDO2_REQUEST_SESSION_KEY, self.client.session)

    @override_settings(DJANGO_FIDO_TWO_STEP_AUTH=False)
    def test_get_by_username(self):
        Authenticator.objects.create(user=self.user, attestation_data=ATTESTATION_OBJECT)

        response = self.client.get(self.url, data={'username': self.user.username})

        self.assertEqual(response.status_code, 200)
        # Check response
        state = self.client.session[FIDO2_REQUEST_SESSION_KEY]
        challenge = websafe_decode(state['challenge'])
        fido2_request = {
            'publicKey': {'rpId': 'testserver',
                          'challenge': base64.b64encode(challenge).decode('utf-8'),
                          'allowCredentials': [{'id': CREDENTIAL_ID, 'type': 'public-key'}],
                          'timeout': 30000}}
        if fido2.__version__ < '0.8':
            fido2_request['publicKey']['userVerification'] = 'preferred'
        self.assertEqual(response.json(), fido2_request)


@override_settings(ROOT_URLCONF='django_fido.tests.urls', TEMPLATES=TEMPLATES,
                   AUTHENTICATION_BACKENDS=['django_fido.backends.Fido2AuthenticationBackend'])
class TestFido2AuthenticationView(TestCase):
    """Test `Fido2AuthenticationView` class."""

    url = reverse_lazy('django_fido:authentication')
    state = {'challenge': AUTHENTICATION_CHALLENGE, 'user_verification': UserVerificationRequirement.PREFERRED}

    def setUp(self):
        self.user = User.objects.create_user(USERNAME, password=PASSWORD)
        self.device = Authenticator.objects.create(user=self.user, credential_id_data=CREDENTIAL_ID,
                                                   attestation_data=ATTESTATION_OBJECT)

    @override_settings(DJANGO_FIDO_TWO_STEP_AUTH=True)
    def test_no_user(self):
        with self.settings(LOGIN_URL='/login/'):
            response = self.client.get(self.url)

        self.assertRedirects(response, '/login/', fetch_redirect_response=False)

    @override_settings(DJANGO_FIDO_TWO_STEP_AUTH=False)
    def test_no_user_one_step(self):
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)

    @override_settings(DJANGO_FIDO_TWO_STEP_AUTH=True)
    def test_get(self):
        session = self.client.session
        session[AUTHENTICATION_USER_SESSION_KEY] = self.user.pk
        session.save()

        response = self.client.get(self.url)

        self.assertContains(response, 'Authenticate a FIDO 2 authenticator')

    @override_settings(DJANGO_FIDO_TWO_STEP_AUTH=True)
    def test_post(self):
        session = self.client.session
        session[AUTHENTICATION_USER_SESSION_KEY] = self.user.pk
        session[FIDO2_REQUEST_SESSION_KEY] = self.state
        session.save()

        post = {'client_data': AUTHENTICATION_CLIENT_DATA, 'credential_id': CREDENTIAL_ID,
                'authenticator_data': AUTHENTICATOR_DATA, 'signature': SIGNATURE}
        with self.settings(LOGIN_REDIRECT_URL='/redirect/'):
            response = self.client.post(self.url, post)

        self.assertRedirects(response, '/redirect/', fetch_redirect_response=False)
        self.assertEqual(get_user(self.client), self.user)
        self.assertQuerysetEqual(Authenticator.objects.values_list('user', 'counter'), [(self.user.pk, 152)],
                                 transform=tuple)
        self.assertNotIn(FIDO2_REQUEST_SESSION_KEY, self.client.session)
        self.assertNotIn(AUTHENTICATION_USER_SESSION_KEY, self.client.session)

    @override_settings(DJANGO_FIDO_TWO_STEP_AUTH=True)
    def test_post_no_session(self):
        session = self.client.session
        session[AUTHENTICATION_USER_SESSION_KEY] = self.user.pk
        session.save()

        post = {'client_data': AUTHENTICATION_CLIENT_DATA, 'credential_id': CREDENTIAL_ID,
                'authenticator_data': AUTHENTICATOR_DATA, 'signature': SIGNATURE}
        response = self.client.post(self.url, post)

        self.assertContains(response, 'Authenticate a FIDO 2 authenticator')
        self.assertEqual(response.context['form'].errors, {NON_FIELD_ERRORS: ['Authentication request not found.']})
        self.assertNotIn(FIDO2_REQUEST_SESSION_KEY, self.client.session)
        self.assertEqual(self.client.session[AUTHENTICATION_USER_SESSION_KEY], self.user.pk)

    @override_settings(DJANGO_FIDO_TWO_STEP_AUTH=True)
    def test_post_invalid_response(self):
        session = self.client.session
        session[AUTHENTICATION_USER_SESSION_KEY] = self.user.pk
        session[FIDO2_REQUEST_SESSION_KEY] = self.state
        session.save()

        post = {'client_data': AUTHENTICATION_CLIENT_DATA, 'credential_id': CREDENTIAL_ID,
                'authenticator_data': AUTHENTICATOR_DATA, 'signature': 'INVALID='}
        response = self.client.post(self.url, post)

        self.assertContains(response, 'Authenticate a FIDO 2 authenticator')
        self.assertEqual(response.context['form'].errors, {NON_FIELD_ERRORS: ['Authentication failed.']})
        self.assertNotIn(FIDO2_REQUEST_SESSION_KEY, self.client.session)
        self.assertEqual(self.client.session[AUTHENTICATION_USER_SESSION_KEY], self.user.pk)

    @override_settings(
        DJANGO_FIDO_TWO_STEP_AUTH=False,
        AUTHENTICATION_BACKENDS=['django_fido.backends.Fido2GeneralAuthenticationBackend'],
        DJANGO_FIDO_AUTHENTICATION_BACKENDS=['django.contrib.auth.backends.ModelBackend'],
    )
    def test_post_one_step(self):
        session = self.client.session
        session[FIDO2_REQUEST_SESSION_KEY] = self.state
        session.save()

        post = {'username': USERNAME, 'password': PASSWORD, 'client_data': AUTHENTICATION_CLIENT_DATA,
                'credential_id': CREDENTIAL_ID, 'authenticator_data': AUTHENTICATOR_DATA, 'signature': SIGNATURE}
        with self.settings(LOGIN_REDIRECT_URL='/redirect/'):
            response = self.client.post(self.url, post)

        self.assertRedirects(response, '/redirect/', fetch_redirect_response=False)
        self.assertEqual(get_user(self.client), self.user)
        self.assertQuerysetEqual(Authenticator.objects.values_list('user', 'counter'), [(self.user.pk, 152)],
                                 transform=tuple)
        self.assertNotIn(FIDO2_REQUEST_SESSION_KEY, self.client.session)
        self.assertNotIn(AUTHENTICATION_USER_SESSION_KEY, self.client.session)

    @override_settings(
        DJANGO_FIDO_TWO_STEP_AUTH=False,
        AUTHENTICATION_BACKENDS=['django_fido.backends.Fido2GeneralAuthenticationBackend'],
        DJANGO_FIDO_AUTHENTICATION_BACKENDS=['django.contrib.auth.backends.ModelBackend'],
    )
    def test_post_one_step_error(self):
        session = self.client.session
        session[FIDO2_REQUEST_SESSION_KEY] = self.state
        session.save()

        post = {'username': USERNAME, 'password': 'wrong_password', 'client_data': AUTHENTICATION_CLIENT_DATA,
                'credential_id': CREDENTIAL_ID, 'authenticator_data': AUTHENTICATOR_DATA, 'signature': SIGNATURE}
        response = self.client.post(self.url, post)

        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.context['form'].errors, {
            '__all__': ['Please enter a correct username and password and use valid FIDO2 security key.'],
        })
        self.assertNotIn(FIDO2_REQUEST_SESSION_KEY, self.client.session)
        self.assertNotIn(AUTHENTICATION_USER_SESSION_KEY, self.client.session)

    @override_settings(
        DJANGO_FIDO_TWO_STEP_AUTH=False,
        AUTHENTICATION_BACKENDS=['django_fido.backends.Fido2GeneralAuthenticationBackend'],
        DJANGO_FIDO_AUTHENTICATION_BACKENDS=['django.contrib.auth.backends.ModelBackend'],
    )
    def test_post_one_step_no_request(self):
        post = {'username': USERNAME, 'password': PASSWORD, 'client_data': AUTHENTICATION_CLIENT_DATA,
                'credential_id': CREDENTIAL_ID, 'authenticator_data': AUTHENTICATOR_DATA, 'signature': SIGNATURE}
        response = self.client.post(self.url, post)

        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.context['form'].errors, {
            '__all__': ['Authentication request not found.'],
        })
        self.assertNotIn(FIDO2_REQUEST_SESSION_KEY, self.client.session)
        self.assertNotIn(AUTHENTICATION_USER_SESSION_KEY, self.client.session)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.21 on 2019-06-20 13:09
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_fido', '0007_attestation_data_not_null'),
    ]

    operations = [
        migrations.AddField(
            model_name='authenticator',
            name='credential_id_data',
            field=models.TextField(null=True),
        ),
    ]



"""django_fido app config."""
from django.apps import AppConfig
from django.utils.translation import gettext_lazy as _

from django_fido.settings import DjangoFidoSettings


class DjangoFidoConfig(AppConfig):
    """django_fido app config."""

    name = 'django_fido'
    verbose_name = _('Django application for FIDO protocol')

    def ready(self):
        """Check configuration."""
        DjangoFidoSettings.check()



# -*- coding: utf-8 -*-
# Generated by Django 1.11.21 on 2019-06-20 13:10
import base64

from django.db import migrations


def fill_credential_ids(apps, schema_editor):
    # We need to use historical model
    Authenticator = apps.get_model("django_fido", "Authenticator")
    db_alias = schema_editor.connection.alias
    for authenticator in Authenticator.objects.all():
        authenticator.credential_id_data = base64.b64encode(authenticator.credential.credential_id).decode('utf-8')
        authenticator.save()


class Migration(migrations.Migration):

    dependencies = [
        ('django_fido', '0008_add_credential_id'),
    ]

    operations = [
        migrations.RunPython(fill_credential_ids, reverse_code=migrations.RunPython.noop, elidable=True),
    ]



#!/usr/bin/python3
# -*- coding: utf-8 -*-
"""Setup script for django_fido."""
from distutils.command.build import build

from setuptools import find_packages, setup
from setuptools.command.sdist import sdist

import django_fido

LONG_DESCRIPTION = open('README.md').read() + '\n\n' + open('CHANGELOG.md').read()
CLASSIFIERS = ['Development Status :: 4 - Beta',
               'License :: OSI Approved :: MIT License',
               'Framework :: Django',
               'Framework :: Django :: 1.11',
               'Framework :: Django :: 2.0',
               'Framework :: Django :: 2.1',
               'Framework :: Django :: 2.2',
               'Framework :: Django :: 3.0',
               'Intended Audience :: Developers',
               'License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)',
               'Operating System :: OS Independent',
               'Programming Language :: Python',
               'Programming Language :: Python :: 3',
               'Programming Language :: Python :: 3.5',
               'Programming Language :: Python :: 3.6',
               'Programming Language :: Python :: 3.7',
               'Programming Language :: Python :: 3.8',
               'Topic :: Internet :: WWW/HTTP',
               'Topic :: Security :: Cryptography',
               'Topic :: Software Development :: Libraries :: Python Modules']
INSTALL_REQUIRES = [
    'Django>=1.11',
    'fido2<0.9.0',
    'sqlparse',  # sqlparse is required for Django < 2.2
    'django-app-settings>=0.7.1',
    'jwcrypto',
    'requests',
    'cryptography',
    'pyopenssl',
]
EXTRAS_REQUIRE = {'quality': ['isort', 'flake8', 'pydocstyle', 'mypy', 'polint'],
                  'test': ['mock', 'responses']}


class custom_build(build):

    sub_commands = [('compile_catalog', None), ('build_js', None)] + build.sub_commands


class custom_sdist(sdist):

    def run(self):
        self.run_command('compile_catalog')
        # sdist is an old style class so super cannot be used.
        sdist.run(self)


setup(name='django-fido',
      version=django_fido.__version__,
      description='Django application for FIDO protocol',
      long_description=LONG_DESCRIPTION,
      long_description_content_type='text/markdown',
      author='Vlastimil Zíma',
      author_email='vlastimil.zima@nic.cz',
      url='https://github.com/CZ-NIC/django-fido',
      packages=find_packages(),
      include_package_data=True,
      python_requires='~=3.5',
      setup_requires=['Babel >=2.3', 'setuptools_webpack'],
      install_requires=INSTALL_REQUIRES,
      extras_require=EXTRAS_REQUIRE,
      keywords=['django', 'fido', 'u2f', 'fido2'],
      classifiers=CLASSIFIERS,
      webpack_output_path='django_fido/static/django_fido/js',
      cmdclass={'build': custom_build, 'sdist': custom_sdist})



# -*- coding: utf-8 -*-
# Generated by Django 1.11.28 on 2020-02-20 04:41
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('django_fido', '0012_authenticator_label'),
    ]

    operations = [
        migrations.AlterField(
            model_name='authenticator',
            name='label',
            field=models.TextField(blank=True, default='', max_length=255),
            preserve_default=False,
        ),
        migrations.AlterUniqueTogether(
            name='authenticator',
            unique_together=set([('user', 'label')]),
        ),
    ]



# Generated by Django 2.2.13 on 2020-07-22 11:25

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_fido', '0014_authenticator_metadata'),
    ]

    operations = [
        migrations.AlterField(
            model_name='authenticatormetadata',
            name='url',
            field=models.URLField(unique=True),
        ),
    ]
